FUNCTION drz_healp_drizzle5,healp_header,fits_header,plot_it=plot_it,fits_inside_out=fits_inside_out,lbl=lbl,allsky=allsky, jd=jd, silent=silent

;+
; NAME:
;    healp_drizzle2
; CATEGORY:
;    Healpix additions of JPBlib
; PURPOSE:
;    retruns a structure allowing drizzle calculations in healpix
; CALLING SEQUENCE:
;    st=healp_drizzle(healp_header,fits_header[,/plot_it])
; INPUT:
;    healp_headere = healpix header
;    fits_header = header of the fits file
; OUTPUT:
;    st = drizzle structure
;    The structure has the following form:
;    st.pixel           = healpix pixel number
;    st.pix_inside      = pointer on the fits pixels numbers intersecting the
;                        healpix pixel
;    st.corner_is_inside=
;    st.surface_inside  = pointer on the normalized surface of the fits pixel
;                         falling inside the healpix pixel 
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    plot_it = if set, plots the polygon intersections
; OUTPUT KEYWORDS:
;    fits_inside_out= if set, outputs a reversed structure similar to
;    st, but indexed on fits pixels.
; SIDE EFFECTS:
;    None
; EXAMPLES:
;    
; RESTRICTIONS:
;    None
; PROCEDURES CALLED:
;    
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard 21/10/2005
;-

@drz_proj_common.com

order=1
double=1

h=fits_header
drz_h2pix,fits_header,/silent
Nx=sxpar(h,'NAXIS1')
Ny=sxpar(h,'NAXIS2')
cdelt1=sxpar(h,'CDELT1')
cdelt2=sxpar(h,'CDELT2')
;apparently, was not used. Removed
;create_coo2,h,c1,c2,/silent

;make healpix vector
Nside=sxpar(healp_header,'NSIDE')
ordering=strtrim(sxpar(healp_header,'ORDERING'),2)
coordsys=strtrim(sxpar(healp_header,'COORDSYS'),2)
Npix=nside2npix(nside)


IF proj EQ 'CSC' THEN BEGIN
    ipix=l64indgen(Npix)
    ;compute all healpix pixel center coordinates
    pix2ang,nside,ipix,theta_ce,phi_ce,ordering=ordering,lbl=lbl
    Nv=n_elements(phi_ce)
ENDIF ELSE BEGIN
    IF keyword_set(allsky) THEN BEGIN
        Nv = Npix
        healp_index = lindgen(Npix)
    ENDIF ELSE BEGIN
        ;=== compute healpix indexes intercepting the fits area
        if not keyword_set(silent) then message,'Computing healpix/fits intersection ...',/info
        healp_index=drz_healp_fits_interpix2(healp_header,fits_header,lbl=lbl,count=count_healpix)
        ;healp_index=[49045503LL] & count_healpix=1L
        ;healp_index=[49128446LL] & count_healpix=1L
        Nv=count_healpix
    ENDELSE
ENDELSE


;stop
;compute index of corners in FITS image
drz_h2pix,h,ipix,jpix,/silent
ijpix=lonarr(Nx*Ny,2)
ijpix(*,0)=ipix
ijpix(*,1)=jpix
ipix=1 & jpix=1
all_pix_nums=reform(drz_ij2index(ijpix,[Nx,Ny]),Nx,Ny)
ijpix=1

;print, 'proj : ', proj 
;stop

one_pix={pixel:0LL, $
         pix_inside:ptr_new(), $
;         corner_is_inside:ptr_new(), $
         surface_inside:ptr_new()}

IF proj EQ 'CSC' THEN BEGIN
    nbface = 6
    face = indgen(6)
    Nface = Nx / 3
    xoffset = [2,2,1,0,0,1]
    yoffset = [1,0,0,0,1,1]
    debx = xoffset * Nface
    finx = debx + Nface-1
    deby = yoffset * Nface
    finy = deby + Nface-1
    
    drizz={geom:replicate(one_pix,Npix),hfits:fits_header,hhealpix:healp_header}
    drizz.geom(*).pixel=lindgen(Npix)
    ;inside_out=replicate(one_pix,Npix)
    ;inside_out(*).pixel=lindgen(Npix) 

ENDIF ELSE BEGIN
    nbface = 1
    debx = [0]
    deby = [0]
    finx = [Nx-1]
    finy = [Ny-1]
ENDELSE

; LOOP ON THE FACES
;******************
FOR iface=0,nbface-1 DO BEGIN

    if not keyword_set(silent) then print, 'Face number '+strtrim(iface,2)
    if not keyword_set(silent) then print, '--------------'

    ;select the healpix pixels covered by the fits image
    ;****************************************************
    if not keyword_set(silent) then print, '    Select the healpix pixels covered'
    th_ce=1
    IF not keyword_set(lbl) THEN lbl=10000L
;    Nv=n_elements(phi_ce)
    Nchuncks=round(Nv/lbl)+1
    index=[0L]
    FOR i=0L,Nchuncks-1 DO BEGIN
        IF proj NE 'CSC' THEN BEGIN
        ;    drz_coo2pix2,cc1,cc2,i_ce,j_ce,ph_ce,th_ce
        ;    ;ind=where((i_ce GE 0 AND i_ce LE Nx-1 AND j_ce GE 0 AND j_ce LE Ny-1) AND th_ce GE 0,count)
        ;    ind=where((i_ce GE -0.5 AND i_ce LE (Nx-0.5) AND j_ce GE -0.5 AND j_ce LE Ny-0.5) AND th_ce GE 0,count)
        ;    If count NE 0 THEN Index=[index,ind+ifrom]
        ENDIF ELSE BEGIN
            ifrom=(i*lbl)<(Nv-1) & ito=(ifrom+lbl-1)<(Nv-1)
            drz_healp_phi_theta2coo,phi_ce(ifrom:ito)*!radeg,theta_ce(ifrom:ito)*!radeg,cc1,cc2,healp_header,fits_header
            drz_coo2pix2,cc1,cc2,i_ce,j_ce,ph_ce,th_ce, face=face_ce
            ind = where(face_ce eq iface, count)
            If count NE 0 THEN Index=[index,ind+ifrom]
            
            display=0
            if display ne 0 then begin
                maptemp = fltarr(nside2npix(nside))
                maptemp(ind) = ind
                mollview, maptemp, /on, /nested, max = nside2npix(nside)-1
                ;stop
            endif
   
        ENDELSE
    ENDFOR
    IF proj NE 'CSC' THEN index=healp_index ELSE index=index(1:*)
    th_ce=0
    i_ce=0 & j_ce=0

;    if not keyword_set(silent) then print, '    Enlarge the selection of healpix pixels'
;    radius=drz_nside2pixsize(nside)*sqrt(2.)
;    index=healp_enlarge(nside,index,radius,ordering=ordering)
    count_healp=n_elements(index)

    display=0
    if display ne 0 then begin
        maptemp = fltarr(nside2npix(nside))
        maptemp(index) = index
        mollview, maptemp, /on, /nested, max = nside2npix(nside)-1
        stop
    endif
    
    ;estimpate surface of healpix and fits pixels
    ;*********************************************
    surf_pix_healp=(drz_nside2pixsize(nside))^2
    surf_pix_fits=abs(cdelt1*cdelt2)
    surf_tot_fits=Nx*Ny*surf_pix_fits

    ;compute healpix corners coordinates
    ;***********************************
    if not keyword_set(silent) then print, '    Compute healpix corners coordinates'
    drz_pix2corners,nside,index,theta_co,phi_co,ordering=ordering
    th_co=1 & ph_co=1
    drz_healp_phi_theta2coo,phi_co*!radeg,theta_co*!radeg,cc1,cc2,healp_header,fits_header
    drz_coo2pix2,cc1,cc2,i_co,j_co,ph_co,th_co, forceface= lonarr(count_healp*4) + iface

    Nmaxpix=2*ceil(surf_pix_healp/surf_pix_fits) + 1
    if keyword_set(allsky) then Nmaxpix=10*ceil(surf_pix_healp/surf_pix_fits) + 1
    Nmaxpix=Nmaxpix<(Nx*Ny)                        ;Since there cannot be more fits pixels than the total number
    pixnums=lon64arr(Nmaxpix)                           ;list of all fits pixels falling into a given healpix pixel
    corner_inside=lonarr(Nmaxpix,4)
    surf=dblarr(Nmaxpix)
    IF proj NE 'CSC' THEN BEGIN
        drizz={geom:replicate(one_pix,count_healp),hfits:fits_header,hhealpix:healp_header}
    ENDIF
    i_offset=[-0.5,+0.5,+0.5,-0.5]
    j_offset=[-0.5,-0.5,+0.5,+0.5]
    tr1=[0,1,2]
    tr2=[0,2,3]
    ;count=100L                  ;for tests

    ; Loop on Healpix pixels
    ;***********************
    if not keyword_set(silent) then begin
        print, '    Loop over healpix pixels'
        iper = 0
    endif
    FOR i=0LL,count_healp-1 DO BEGIN  
;        IF healp_index(i) EQ 48656328LL THEN stop

        ;This is the healpix pixel number
        IF proj NE 'CSC' THEN drizz.geom(i).pixel=index(i) 

        ;define region of FITS image containing the healpix pixel
        imin=fix(min(i_co(i,*)))> debx(iface)
        jmin=fix(min(j_co(i,*)))> deby(iface)
        imax=ceil(max(i_co(i,*)))< finx(iface)
        jmax=ceil(max(j_co(i,*)))< finy(iface)

        ;Find out if the corners of the fits pixels are inside the healpix pixel
        i_polygon=reform(i_co(i,*)) & j_polygon=reform(j_co(i,*))

        display = 0
        if display eq 1 then begin
            print, 'healpix pixel numero : ',i
            map_hpx = lonarr(12L*nside^2)  - 10
            map_hpx(index) = index
            map_hpx(index(i)) = 12L*nside^2
            mollview, map_hpx, /on, /nested, min=-1, max = 12L*nside^2-1
            window, 1
        endif

      IF keyword_set(jd) THEN BEGIN
        pixnums=polyfillaa(i_polygon+0.5,j_polygon+0.5,Nx,Ny,areas=surf)
        IF isarray(pixnums) THEN BEGIN
          iv=n_elements(pixnums)
        ENDIF ELSE BEGIN
          iv=0L
        ENDELSE
      ENDIF ELSE BEGIN

        iv=0L
        FOR ip=imin,imax DO BEGIN
            FOR jp=jmin,jmax DO BEGIN
                i_fits_corners=ip+i_offset
                j_fits_corners=jp+j_offset
                is_inside2=inside(i_fits_corners, j_fits_corners, i_polygon, j_polygon)
                ind2=where(is_inside2 EQ 1,c2) ;fits corners inside the healpix pixel
                is_inside1=inside(i_polygon,j_polygon,i_fits_corners, j_fits_corners)
                ind1=where(is_inside1 EQ 1,c1) ;healpix corners inside the fits pixel
                is_inside3 =inside(ip,jp,i_polygon,j_polygon)
                ind3=where(is_inside3 EQ 1,c3) ;fits center inside the healpix pixel
                pixnums(iv)=all_pix_nums(ip,jp)
                corner_inside(iv,*)=is_inside1
                vold = iv
                surf_temp = drz_polygons_inter(i_polygon,j_polygon,i_fits_corners,j_fits_corners, $
                                               is_inside1,is_inside2,order=order,plot_it=plot_it,double=double)
;                print,'++++',c1,c2,c3,ip,jp,surf_temp,healp_index(i)
                IF c2 EQ 4 THEN BEGIN 
                    surf(iv)=1. & iv=iv+1
;                    IF ip EQ 443 AND jp EQ 352 THEN BEGIN
;                      print,'====',c1,c2,c3,ip,jp,surf(iv-1),healp_index(i)
;                    ENDIF
                ENDIF ELSE BEGIN
;                    IF NOT ((c2 EQ 0) AND (c1 EQ 0) AND (c3 EQ 0))  THEN BEGIN
;                        stop
                        surf_temp = drz_polygons_inter(i_polygon,j_polygon,i_fits_corners,j_fits_corners, $
                                                   is_inside1,is_inside2,order=order,plot_it=plot_it,double=double)
                        IF surf_temp NE 0 THEN BEGIN
                            surf(iv) = surf_temp
                            iv=iv+1
;                           print,'====',c1,c2,c3,ip,jp,surf(iv-1),healp_index(i)
                        ENDIF
;                    ENDIF
                ENDELSE
                if display eq 1 then begin
                    print, 'fits pixel numero', ip, jp
                    surf2 = drz_polygons_inter(i_polygon,j_polygon,i_fits_corners,j_fits_corners, $
                                           is_inside1,is_inside2, $
					   order=order,plot_it=plot_it,double=double)
                    plot, -i_fits_corners, j_fits_corners , psym = 2, xr = [-4,4], yr=[-4,4], $
                          title='detecion = '+strtrim(iv-vold,2)+' surf = '+strtrim(surf2,2)
                    oplot, -i_fits_corners, j_fits_corners
                    oplot, shift(-i_fits_corners,1), shift(j_fits_corners,1)
                    oplot, -i_co(i,*), j_co(i,*) , psym = 4      
                    oplot, -i_co(i,*), j_co(i,*), col = 250     
                    oplot, shift(-i_co(i,*),1), shift(j_co(i,*),1), col = 250
                    print, '   fits in healpix :',strtrim(c2,2), ',  soit : ', is_inside2
                    p = where(is_inside2 eq 1, ct)
                    if ct ne 0 then oplot, -i_fits_corners(p), j_fits_corners(p) , psym = 2, col= 50
                    print, '   healpix in fits : ',strtrim(c1,2), ',  soit :', is_inside1
                    p = where(is_inside1 eq 1, ct)
                    if ct ne 0 then oplot, -i_co(i,p), j_co(i,p) , psym = 4, col= 50
                    stop
                endif

            ENDFOR
         ENDFOR
      ENDELSE
;        stop
        IF iv NE 0 THEN BEGIN
            IF proj NE 'CSC' THEN BEGIN
		tmp = pixnums(0:iv-1)
		ptr_free, drizz.geom(i).pix_inside
                drizz.geom(i).pix_inside=ptr_new(tmp)
		tmp = surf(0:iv-1)
		ptr_free, drizz.geom(i).surface_inside
                drizz.geom(i).surface_inside=ptr_new(tmp)
            ENDIF ELSE BEGIN
                IF ptr_valid(drizz.geom(index(i)).pix_inside) THEN BEGIN
                    pixnums_old=*(drizz.geom(index(i)).pix_inside)
		    ptr_free, drizz.geom(index(i)).pix_inside
		    pixnums_new = [pixnums_old,pixnums(0:iv-1)]
                    drizz.geom(index(i)).pix_inside = ptr_new(pixnums_new)

                    surf_old = *(drizz.geom(index(i)).surface_inside)
		    ptr_free, drizz.geom(index(i)).surface_inside
		    surf_new = [surf_old,surf(0:iv-1)]
                    drizz.geom(index(i)).surface_inside = ptr_new(surf_new)
                ENDIF ELSE BEGIN
		    tmp = pixnums(0:iv-1)
                    drizz.geom(index(i)).pix_inside=ptr_new(tmp)
		    tmp = surf(0:iv-1)
                    drizz.geom(index(i)).surface_inside=ptr_new(tmp)
                ENDELSE
            ENDELSE
        ENDIF
;        stop
        if not keyword_set(silent) then begin
            if ((i+1) mod (count_healp/10)) eq 0 then begin
                iper = iper+10
                print, '       '+strtrim(iper,2)+' %'
            endif
        endif

        ; message,'Done healpix pixel:'+strtrim(i,2)+'/'+strtrim(count_healp-1),/info
    ENDFOR

    display = 0
    if display eq 1 then begin
        st = drizz.geom
        for i=0,n_elements(st.pixel)-1 do begin
            if ptr_valid(st(i).pix_inside) then begin
                print, strtrim(st(i).pixel,2)+' : ',*(st(i).pix_inside)
            endif
        endfor
        stop
    endif

ENDFOR


;== If necessary, computes the inversed structure (ie fits pixel based)
;***********************************************************************
IF keyword_set(fits_inside_out) THEN BEGIN
  ind_good=where(ptr_valid(drizz.geom.pix_inside),count_good)
  Npix_fits=Nx*Ny
  fits_inside_out=replicate(one_pix,Npix_fits)
  FOR i=0L,Npix_fits-1 DO BEGIN
    fits_inside_out(i).pixel=i
    all_pix=[1L] & all_surf=[!drz_undef]
    FOR j=0L,count_good-1 DO BEGIN
      ind=where(*drizz.geom(ind_good(j)).pix_inside EQ i,count)
      IF count NE 0 THEN BEGIN
        all_pix=[all_pix,drizz.geom(ind_good(j)).pixel]
        all_surf=[all_surf,(*drizz.geom(ind_good(j)).surface_inside)(ind(0))]
      ENDIF
    ENDFOR
    tmp = all_pix(1:*)
    fits_inside_out(i).pix_inside=ptr_new(tmp)
    tmp = all_surf(1:*)
    fits_inside_out(i).surface_inside=ptr_new(tmp)
  ENDFOR
ENDIF


RETURN, drizz

END
